:- use_package([assertions]).

%% CIAO-specific
:- use_module(library(format)).
:- use_module(library(ttyout)).

%----------------------------------------------------------------------
% Format a paper for latex itemize, english
%----------------------------------------------------------------------

:- ensure_loaded(support).

% Format common info

pp_papers_latex_eng(Papers) :-
%% 	format(user_error,"[Writing out papers",[]),ttyflush,
	do_pp_papers_latex_eng(Papers,0).

do_pp_papers_latex_eng([],_Count).
%% 	format(user_error," done. ~w entries processed.]~n",[Count]),ttyflush.
do_pp_papers_latex_eng([H|T],Count) :-
        pp_paper_latex_eng(H),
	!,
 	format(user_error,".",[]),ttyflush,
	NCount is Count + 1,
	do_pp_papers_latex_eng(T,NCount).
do_pp_papers_latex_eng([prolog_paper(_Y,_M,K,_L)|T],Count) :- 
	format(user_error,"[ERROR: could not format ~w",[K]),
	do_pp_papers_latex_eng(T,Count).

pp_paper_latex_eng(prolog_paper(_Year,_Month,Keyword,List)) :-
	member([butype,Type],List),
	format('\\item ',[]),
        print_item(author,List,Keyword,". "),
	format('{\\em ',[]),
	print_item(title,List,Keyword,"."),
	format('}~n',[]),
	print_special_info(Type,List,Keyword),
	print_item_optional(publisher,List,Keyword,", "),
	print_item_optional(series,List,Keyword,", "),
	(  (Type \== "techreport", Type \== "article")
        -> print_item_optional(number,List,Keyword,", ")
        ;  true ),
	print_item_optional(edition,List,Keyword,", "),
	print_item_optional(institution,List,Keyword,", "),
	print_item_optional(organization,List,Keyword,", "),
	( Type = "article"
        -> print_item_optional(month,List,Keyword," ")
        ;  print_item(month,List,Keyword," ")),
        print_item(year,List,Keyword,"."),
        print_item_optional(note,List,Keyword,"."),
	format('~n',[]),
	ttyflush.

% Format a field

print_item_optional(Item,List,Keyword,Sep) :-
	print_item_common(Item,List,Keyword,Sep),
	!.
print_item_optional(_Item,_List,_Keyword,_Sep).

print_item(Item,List,Keyword,Sep) :-
	print_item_common(Item,List,Keyword,Sep),
	!.
print_item(Item,_List,Keyword,_Sep) :-
	format(user_error,'[WARNING: No ~a info in DB for ~a]~n',
                           [Item,Keyword]).

print_item_common(pages,List,Keyword,Sep):-
	get_page_number(Keyword,List,Number,In,En),
	!,
	(In == [] -> 	
           ( number(Number) ->
	     format('~w',[Number]),
	     format(' pages~s~n',[Sep])
	   ; true )
	;  format(' pages ~d--~d~s~n',[In,En,Sep])
	). 
print_item_common(title,List,Keyword,Sep) :-
	member([title,Title],List),
	!,
	(  ( member([url,URL],List); 
	     ps_exists(Keyword,URL) )
	-> add_html2latex_link(URL,Title,LTitle)
        ;  LTitle = Title),
	do_print_item(title,LTitle),
	format("~s~n",[Sep]).
print_item_common(booktitle,List,_Keyword,Sep) :-
	member([booktitle,Title],List),
	!,
	(  member([bookurl,URL],List) 
	-> add_html2latex_link(URL,Title,LTitle)
        ;  LTitle = Title),
	do_print_item(booktitle,LTitle),
	format("~s~n",[Sep]).
print_item_common(Item,List,_Keyword,Sep) :-
	member([Item,AList],List),
	!,
	do_print_item(Item,AList),
	format("~s~n",[Sep]).

do_print_item(I,[A]):-
	(I = author; I = editor),
	!,
        possibly_add_person_url(A,URLA),
	format('~s',[URLA]).
do_print_item(I,[A|T]):-
	(I = author; I = editor),
	!,
        possibly_add_person_url(A,URLA),
	format('~s, ',[URLA]),
        do_print_item(I,T).
do_print_item(_,A):-
	format('~s',[A]).

possibly_add_person_url(A,URLA) :-
	people_url_info_available(yes),
	add_url(A,URLA).
possibly_add_person_url(A,A).

% Format specialized info

print_special_info("manual",List,Keyword) :- !,
	print_special_info("book",List,Keyword).
print_special_info("proceedings",List,Keyword) :- !,
	print_special_info("book",List,Keyword).
print_special_info("editedbook",List,Keyword) :- !,
	print_special_info("book",List,Keyword).
print_special_info("book",List,Keyword):- 
	!,
	print_item_optional(pages,List,Keyword,", "). 

print_special_info("workshop",List,Keyword) :- !,
	print_item(booktitle,List,Keyword,", "),
	print_item_optional(pages,List,Keyword,", ").

print_special_info("incollection",List,Keyword) :- !,
	print_special_info("inproceedings",List,Keyword).
print_special_info("invited",List,Keyword) :- !,
	print_special_info("inproceedings",List,Keyword).
print_special_info("inproceedings",List,Keyword) :- !,
	print_item(booktitle,List,Keyword,", "),
	print_item(pages,List,Keyword,", ").

print_special_info("techreport",List,Keyword) :- !,
	print_item(number,List,Keyword,", "),
	print_item_optional(pages,List,Keyword,", "). 

print_special_info("article",List,Keyword) :- !,
	print_item(journal,List,Keyword,", "),
        ( member([volume,_],List),!,
          format('Vol. ',[]),
	  print_item(volume,List,Keyword,", Num. "),
	  print_item(number,List,Keyword,", "),
          print_item(pages,List,Keyword,", ")
        ;
          format(' in press, ',[]) 
        ).

print_special_info("mastersthesis",List,Keyword) :- !,
        format("Ms.\\ Thesis, ",[]),
        print_item(school,List,Keyword,", "),
	print_item_optional(address,List,Keyword,", "),
	print_item_optional(pages,List,Keyword,", "). 
print_special_info("phdthesis",List,Keyword) :- !,
        format("Ph.D.\\ Thesis, ",[]),
        print_item(school,List,Keyword,", "),
	print_item_optional(address,List,Keyword,", "),
	print_item_optional(pages,List,Keyword,", "). 

print_special_info(Type,_List,Keyword) :- 
	format(user_error,'[WARNING: Don\'t know how to format ~s in ~a]~n',
                           [Type,Keyword]).
	
%----------------------------------------------------------------------

%% For SICStus
comment(_,_).

:- comment(version_maintenance,dir(version)).

:- comment(version(0*4+34,1999/09/02,21:44*32+'MEST'), "Now 'note'
   field is included in LaTeX output (useful for 'To Appear', etc.).
   (Manuel Hermenegildo)").

:- comment(version(0*4+24,1999/03/16,14:23*51+'MET'), "Changed
   syntax/1 to use_package/1 for 0.9.  (Manuel Hermenegildo)").

:- comment(version(0*3+0,1998/06/19,10:18*10+'MET DST'), "Changed how
   paper urls are added: look for either a url in the bib database or
   the existence of a ps file (@bf{with the same name as the keyword
   of the bib entry}) in a public directory, defined in a fact
   ps_papers_dir_url/2.  (Manuel Hermenegildo)").


%----------------------------------------------------------------------

