%% :- module(support,[sort_papers/3]).

:- use_package([assertions]).

%% CIAO Specific
:- use_module(library(format)).
:- use_module(library(dynamic),[assert/1]).

:- comment(module,"This library contains miscellaneous support
   predicates.").

%-----------------------------------------------------------------------------
:- pred butype_description(Butype,Text) # "@var{Text} is the textual
   description of @var{Butype}. This description should be short (will
   probably be used as a section title).".
%-----------------------------------------------------------------------------

%% 1 * Refereed/serious publications:
%% ----------------------------------
%% - Only refereed articles published in a journal:
butype_description("article","Articles in Refereed Journals").
%% - Only refereed articles published in conference proceedings. 
butype_description("inproceedings","Articles in Refereed Conferences").
%% - Books >with editor<, including formal proceedings
butype_description("book","Books and Monographs").
%% 2 * Unrefereed, but important publications:
%% -------------------------------------------
%% - Invited papers. These have ``inproceedings'' bibtex
%%   type but ``invited'' butype. 
butype_description("invited","Invited Papers and Tutorials").
%% - Paper in book >with editor<
butype_description("incollection","Articles in Books").
%% - Workshop papers. These have ``inproceedings'' bibtex
%%   type but ``workshop'' butype.
butype_description("workshop","Publications in Workshops").
butype_description("techreport","Technical Reports and Manuals").
%% These are sometimes collapsed to the above:
butype_description("manual","Technical Reports and Manuals").
butype_description("editedbook","Edited Books and Proceedings").
butype_description("mastersthesis","Master's Theses").
butype_description("phdthesis","Ph.D. Theses").
%% - No need for publisher, etc. -> e.g., workshop proceedings?:
butype_description("proceedings","Proceedings").
%% - Invited talks and tutorials at major conferences, when
%%   there is no paper associated with them (in that case
%%   they are ``invited''). These have ``inproceedings'' bibtex  
%%   type but ``invitedtalk'' butype.
butype_description("invitedtalk","Invited Talks and Tutorials").
%% Accepts anything (e.g., internet postings)
butype_description("misc","Miscellaneous").
%% Note required! For manuscripts...
butype_description("unpublished","Other Documents").

%-----------------------------------------------------------------------------
:- pred sort_papers(+list,-list,+Type) # "Sort papers. @var{Type}
        determines which fields will be used for the comparison.".
%-----------------------------------------------------------------------------

sort_papers([],[],_Type).
sort_papers([X|L],R,Type) :-
	meta_partition(L,X,L1,L2,Type),
	sort_papers(L1,SL1,Type),
	sort_papers(L2,SL2,Type),
	append(SL1,[X|SL2],R).

meta_partition([],_,[],[],_Type).
meta_partition([X|L],Y,[X|L1],L2,Type) :-
	meta_test(Type,X,Y),
	!,
	meta_partition(L,Y,L1,L2,Type).
meta_partition([X|L],Y,L1,[X|L2],Type) :-
	meta_partition(L,Y,L1,L2,Type).

%-----------------------------------------------------------------------------
:- comment(meta_test/3, "Different comparisons of bib fields in
           papers.  Papers assumed to be in a standard format.").
%-----------------------------------------------------------------------------

:- pred meta_test(^main_author,Paper1,Paper2) # "Main author of
        @var{Paper1} precedes alphabetically main author of
        @var{Paper2}.".

meta_test(main_author,X,Y) :-
	X=prolog_paper(_KeywordX,ListX),
	Y=prolog_paper(_KeywordY,ListY),
	member_check([author,[MainAX|_]],ListX),
	member_check([author,[MainAY|_]],ListY),
        delete_element(MainAX,T1X,0' ),
        delete_element(MainAY,T1Y,0' ),
        delete_element(T1X,T2X,0'{),
        delete_element(T1Y,T2Y,0'{),
        delete_element(T2X,T3X,0'}),
        delete_element(T2Y,T3Y,0'}),
        % Get rid of initials... (up to 3 >different< ones -- 
        % here is where a regexp would be better)
        replace_list(T3X,T4X,[_,0'.],""),
        replace_list(T4X,T5X,[_,0'.],""),
        replace_list(T5X,AuthXS,[_,0'.],""),
        replace_list(T3Y,T4Y,[_,0'.],""),
        replace_list(T4Y,T5Y,[_,0'.],""),
        replace_list(T5Y,AuthYS,[_,0'.],""),
	% format(user_error,'comparing ~s and ~s ~n',[AuthXS,AuthYS]),
        name(AuthX,AuthXS),
        name(AuthY,AuthYS),
	!,
        AuthX @< AuthY.
	% format(user_error,"TEST SUCCEEDED~n",[]), ttyflush.

:- pred meta_test(^year_month,Paper1,Paper2) # "Publication date of
        @var{Paper1} precedes that of @var{Paper2}.".

meta_test(year_month,X,Y) :-
	X=prolog_paper(YearXS,MonthXS,_,_),
	Y=prolog_paper(YearYS,MonthYS,_,_),
	[Char]=" ",
	delete_element(YearXS,NYearXS,Char),
	delete_element(YearYS,NYearYS,Char),
	delete_element(MonthXS,NMonthXS,Char),
	delete_element(MonthYS,NMonthYS,Char),
	month_to_number(NMonthXS,NumMonthXS),
	month_to_number(NMonthYS,NumMonthYS),
	append(NYearXS,NumMonthXS,RNYearXS),
	append(NYearYS,NumMonthYS,RNYearYS),
	name(YearX,RNYearXS),
	name(YearY,RNYearYS),
        YearX @< YearY.

month_to_number("January","01"):- !.
month_to_number("February","02"):- !.
month_to_number("March","03"):- !.
month_to_number("April","04"):- !.
month_to_number("May","05"):- !.
month_to_number("June","06"):- !.
month_to_number("July","07"):- !.
month_to_number("August","08"):- !.
month_to_number("September","09"):- !.
month_to_number("October","10"):- !.
month_to_number("November","11"):- !.
month_to_number("December","12"):- !.
month_to_number(_,"00"). % This can apply when month is not known

%-----------------------------------------------------------------------------
% URL database access / handling
%-----------------------------------------------------------------------------

add_html2latex_link(RURL,Text,LinkedText) :-
	replace_element(RURL,URL,0'~,"\\~{}"),
	append("\\htmladdnormallink{",Text,T1),
	append(T1,"}{",T2),
	append(T2,URL,T3),
	append(T3,"}",LinkedText).

add_html2latex_email(EMAIL,Text,TextAndE) :-
	append(Text," \\htmladdnormallink{
\\htmladdimg{http://www.clip.dia.fi.upm.es/images/email\\_blue\\_small.gif}}{",
              T1),
	append(T1,"mailto:",T2),
	append(T2,EMAIL,T3),
	append(T3,"}",TextAndE).

add_url(Author,URLAuthor) :-
	fix_name(Author,PlainName),
	find_url(Author,PlainName,URLAuthor).

:- dynamic url_not_found_reported/1.

find_url(Author,PlainName,URLAuthor) :-
	url_db(PlainName,HTTP,EMAIL),
	!,
        % HTTP is preferred...
  	(  HTTP = "" 
	-> RURL = [0'm,0'a,0'i,0'l,0't,0'o,0':|EMAIL] 
        ;  RURL = HTTP),
	add_html2latex_link(RURL,Author,URLAuthor).
%% Uncomment this to add also an email button (ugly):
%%   	(  EMAIL \== "" 
%% 	-> add_html2latex_email(EMAIL,URLAuthor1,URLAuthor)
%%         ;  URLAuthor1 = URLAuthor).
find_url(Author,PlainName,Author) :-
	url_not_found_reported(PlainName), %% Error already reported...
	!.
find_url(Author,PlainName,Author) :-
	assert(url_not_found_reported(PlainName)),
	format(user_error,'[WARNING: URL not found for ~s]~n', [PlainName]).

fix_name(Name,PlainName) :- 
	delete_element(Name,T1,0'\\),
	replace_element(T1,T2,0'~," "),
	delete_element(T2,T3,0'\'),
	delete_element(T3,T4,0'{),
	delete_element(T4,T5,0'}),
	delete_element(T5,T6,0' ),
	delete_element(T6,T7,10), % Newlines...
	delete_element(T7,T8,0'-), 
	delete_element(T8,PlainName,0'}).

:- pred ps_exists(Keyword,URL) :: atm * string ; "Checks for the
        existence of a ps file with the name @var{Keyword}.ps or
        @var{Keyword}.ps.gz in a public directory, defined in a fact
        ps_papers_dir_url/2.".

ps_exists(Keyword,URL) :-
	ps_papers_dir_url(Dir,DirURL),
	name(Keyword,KeywordS),
	append(Dir,   [0'/ | KeywordS],FileBase),
	append(DirURL,[0'/ | KeywordS],URLBase),
	nofileerrors,
	( try_to_open(FileBase,URLBase,".ps",URL)
	; try_to_open(FileBase,URLBase,".ps.gz",URL) ),
	fileerrors.
ps_exists(_Keyword,_URL) :-
	fileerrors,
%%	format(user_error,"[WARNING: No ps file found for ~w]~n",[Keyword]),
	fail.

try_to_open(FileBase,URLBase,Suffix,URL) :-
	append(FileBase,Suffix,FileNameS),
	name(FileName,FileNameS),
	open(FileName,read,Stream),
	close(Stream),
	append(URLBase,Suffix,URL).
	
:- multifile ps_papers_dir_url/2.
:- dynamic   ps_papers_dir_url/2.

:- pred ps_papers_dir_url(Dir,URL) :: string * string

   ; "@var{Dir} is a directory containing .ps or .ps.gz
     files. @var{URL} is the WWW address of that directory.".

%-----------------------------------------------------------------------------
% get beginpage and endpage and/or number of pages
%-----------------------------------------------------------------------------

get_page_number(_,List,Number,In,En) :-
	member_check([beginpage,Init],List), !,
	name(In,Init),
	member_check([endpage,End],List),
	name(En,End),
	Number is (En - In) + 1.
get_page_number(_,List,Number,[],[]) :-
	member_check([npages,Init],List), !,
	name(Number,Init).
get_page_number(K, _L, 'N/A', [], []):-
        format(user_error, 
	       "[WARNING: Page number not available on entry ~w]~n",[K]).

%-----------------------------------------------------------------------------
% Month translation (English/Spanish)
%-----------------------------------------------------------------------------

spanish_month("January","Enero"):- !.
spanish_month("February","Febrero"):- !.
spanish_month("March","Marzo"):- !.
spanish_month("April","Abril"):- !.
spanish_month("May","Mayo"):- !.
spanish_month("June","Junio"):- !.
spanish_month("July","Julio"):- !.
spanish_month("August","Agosto"):- !.
spanish_month("September","Septiembre"):- !.
spanish_month("October","Octubre"):- !.
spanish_month("November","Noviembre"):- !.
spanish_month("December","Diciembre"):- !.
spanish_month(A,A).


%-----------------------------------------------------------------------------
% String x List of Strings
%-----------------------------------------------------------------------------

is_an_author(Name,[String|_Rest]) :-
%%	sublist(Name,String), %% Wrong!
	list_contained(Name,String),
	!.
is_an_author(Name,[_String|Rest]) :-
	is_an_author(Name,Rest).

%-----------------------------------------------------------------------------
% Miscellaneous other support predicates
%-----------------------------------------------------------------------------

% InputList x OutputList x InputSubListA x InputSublistB
% OutputList is InputList with InputSublistA replaced by InputSublistB
% (ideally, we should add regexps)
replace_list([],[],_,_).
replace_list(IL,OL,X,Y) :-
	X = [_|_],
        prefix_rest(X,IL,RIL),
	!,
	replace_list(RIL,NOL,X,Y),
        append(Y,NOL,OL).
replace_list([Z|L],[Z|NL],X,Y) :-
        replace_list(L,NL,X,Y).

% InputPrefix x InputList x List
% InputPrefix is a prefix of InputList and List is the rest.
prefix_rest([], R, R).
prefix_rest([X|PT], [X|T], R) :-
	prefix_rest(PT, T, R).

% ... Now, several specializations of the above...

% InputList x OutputList x Char
delete_element([],[],_).
delete_element([X|L],NL,X) :- 
	!,
	delete_element(L,NL,X).
delete_element([Y|L],[Y|NL],X) :- 
	delete_element(L,NL,X).

% InputList x OutputList x Char x List
replace_element([],[],_,_).
replace_element([X|L],R,NX,Y) :- 
	X==NX,
	!,
	replace_element(L,NL,X,Y),
	append(Y,NL,R).
replace_element([Z|L],[Z|NL],X,Y) :- 
	replace_element(L,NL,X,Y).

% ISO escaping...
% *** change to use replace element
prolog_escape([],[]).
prolog_escape([0'\\|T],[0'\\,0'\\|NT]) :- 
	!,
	prolog_escape(T,NT).
prolog_escape([0'"|T],[0'\\,0'"|NT]) :- 
	!,
	prolog_escape(T,NT).
prolog_escape([Y|L],[Y|NL]) :- 
	prolog_escape(L,NL).

delete_spaces([],[]).
delete_spaces([9|R],RD) :-
	!,
	delete_spaces(R,RD).
delete_spaces([10|R],RD) :-
	!,
	delete_spaces(R,RD).
delete_spaces([32|R],RD) :-
	!,
	delete_spaces(R,RD).
delete_spaces([X|R],[X|RD]) :-
	!,
	delete_spaces(R,RD).

% 32 is space
delete_extra_spaces([],[]).
delete_extra_spaces([32,32|T],NT) :-
	!,
	delete_extra_spaces([32|T],NT).
delete_extra_spaces([X|T],[X|NT]) :-
	delete_extra_spaces(T,NT).

list_contained(List1, List2) :-
	append(X,_,List2),
	append(_,List1,X).

sublist(List, List).
sublist(Sub, [Head|Tail]) :- sublist_(Tail, Head, Sub).

sublist_(Sub, _, Sub).
sublist_([Head|Tail], _, Sub) :- sublist_(Tail, Head, Sub).
sublist_([Head|Tail], X, [X|Sub]) :- sublist_(Tail, Head, Sub).

nreverse([],[]).
nreverse([X|L0],L) :- 
	nreverse(L0,L1), 
	append(L1,[X],L).

append([],L,L).
append([X|L1],L2,[X|L3]) :- 
	append(L1,L2,L3).

% Watch out: member_check, really!
member_check(X,[X|_]) :- !.
member_check(X,[_|T]) :- member_check(X,T).

write_strlist([],_Separator,End,Stream) :-
	format(Stream,"~s",End).
write_strlist([Y|RY],Separator,End,Stream) :-
	format(Stream,"~s~s",[Y,Separator]),
	write_strlist(RY,Separator,End,Stream).

to_lower_case([],[]).
to_lower_case([H|T],[NH|NT]) :-
	is_upper_case(H),
	!,
	NH is H+32,
	to_lower_case(T,NT).
to_lower_case([H|T],[H|NT]) :-
	to_lower_case(T,NT).

add_capital_bracing([],[]).
add_capital_bracing([C|T],[0'{,C|NT]) :-
	is_upper_case(C),
	!,
	continue_uppercase(T,NT).
add_capital_bracing([C|T],[C|NT]) :-
	add_capital_bracing(T,NT).


continue_uppercase([],[0'}]).
continue_uppercase([C|T],[125,C|NT]) :-
	is_lower_case(C),
	!,
	add_capital_bracing(T,NT).
continue_uppercase([C|T],[C|NT]) :-
	continue_uppercase(T,NT).

is_upper_case(Ch) :-
	Ch >= 65, Ch =< 90.

is_lower_case(Ch) :-
	Ch >= 97, Ch =< 122.

%----------------------------------------------------------------------
:- comment(version_maintenance,dir(version)).

:- comment(version(0*4+35,1999/09/02,21:45*57+'MEST'), "@tt{\"}
   characters within bib entry fields now supported, as needed for
   umlaut, etc.  (Manuel Hermenegildo)").

:- comment(version(0*4+21,1999/03/16,14:22*35+'MET'), "Changed
   syntax/1 to use_package/1 for 0.9.  (Manuel Hermenegildo)").

:- comment(version(0*4+16,1998/10/27,12:25*08+'EST'), "Errors on urls
   not found now given only once per run.  (Manuel Hermenegildo)").

:- comment(version(0*4+4,1998/10/18,13:22*19+'MET DST'), "Now running
   natively in CIAO Prolog. Added some more comments. (Manuel
   Hermenegildo)").
%----------------------------------------------------------------------
